--/////////////////////////////////////////////////////////////////////////////
--// Name:        edbscript.e
--// Purpose:     EuScript initialization for Euphoria Database Browser
--// Author:      Matt Lewis
--// Modified by:
--// Created:     2003-08-01
--// Copyright:   (c) Matt Lewis
--// Licence:     See documentation
--/////////////////////////////////////////////////////////////////////////////
--include win32lib.ew
with trace
include euscript.e
include eusql.e
include xControls.ew
include eugrid.ew as g
include edbform.ew as f

function EGW_SetCurrentCell( integer grid, atom row, atom col )
    sequence cols
    object void

    deselect_grid( grid )
    select_grid( grid, row )
    return 1

end function


function EGW_ScrollToCell( integer grid, atom row, atom col )
    sequence cols
    cols = EGW_EnumColumns( grid )
    return g:EGW_ScrollToCell( grid, row, cols[col] )
end function

constant add_these = {

    { "value",                  EUFUNC, 1 },

--/topic Scripting
--/proc Win32Lib
--/desc Routines from Win32Lib
--
--These routines from Win32Lib may be used within a form's script.
--Please refer to the Win32Lib documentation for full descriptions
--of the routines.
--<ul>
--/li /b addItem( integer id, sequence text )  Add an item to a list or combo
--/li /b eraseItems( integer id )  Erase all items from a list or combo
--/li /b getText( integer id )  Get the selected text from a control
--/li /b or_all( sequence values ) Bitwise or operation over multiple values
--/li /b repaintWindow ( window ) Force window to be entirely repainted.
--/li /b setFocus( integer control ) Sets the focus to the specified control
--/li /b setIndex( integer id, integer index ) Set the selected item in a list or combo
--/li /b setText( integer id, sequence text ) Set the text for a control
--/li /b setHint( object control, object text ) Set the tooltip text for a control. 


--</ul>

    -- Win32Lib
    { "addItem",                PROC,   2 },
    { "or_all",                 EUFUNC, 1 },
    { "eraseItems",             PROC,   1 },
    { "getText",                EUFUNC, 1 },
    { "setText",                PROC,   2 },
    { "message_box",            EUFUNC, 3 },
    { "setFocus",               PROC,   1 },
    { "setIndex",               PROC,   2 },
    { "setHint",                PROC,   2 },
    { "repaintWindow",          PROC,   1 },

--/topic Scripting
--/proc xControls
--/desc Routines from xControls
--
--These routines from xControls may be used within a form's script.
--Please refer to the <a href="Geometry.htm">xControls</a> documentation for full descriptions
--of the routines.
--<ul>
--/li /b manage( GMID, ChildID, {%,#}, {%,#}, {%,#}, {%,#} )  Sets up a control to be managed.
--</ul>
--Note that the GMID for a form will be pre-defined as /b geo_formname.

    -- xControls
    { "manage",                 PROC,   6 },

--/topic Scripting
--/proc Miscellaneous Routines
--/desc Routines from various include files
--
--These routines come from various files included by EDB, and may be
--used from within a form's script.
--<ul>
--/li message_box( sequence message, sequence caption, atom flags )
--/li pretty_sprint( object x )  Formats an object to be printed
--</ul>
    
    -- sprint.e
    { "pretty_sprint",          EUFUNC, 1 },
    
    -- EuGrid.ew
    { "EGW_GetDataCellValue",   EUFUNC, 3 },
    { "EGW_SetCurrentCell",         EUFUNC, 3 }, -- calls EGW_SetCurrentCell
    { "EGW_GetCurrentCell",     EUFUNC, 1 },
    { "EGW_GetDataCellValue",   EUFUNC, 3 },
    { "EGW_LoadData",           EUFUNC, 3 },
    { "EGW_UnloadData",         EUFUNC, 1 },
    { "EGW_ScrollToCell",       EUFUNC, 3 },


--/topic Scripting
--/proc EuGrid
--/desc Routines from EuGrid
--
--These routines from EuGrid may be used within a form's script.
--Please refer to the <a href="EuGrid.htm">EuGrid</a> documentation for full descriptions
--of the routines, however, you should always refer to the column index (as
--counted from left to right in the grid) rather than the column id (which can 
--normally be retrieved from the EuGrid function EGW_EnumColumns).  EDB will
--automatically convert the value for those functions that require it.
--<ul>
--/li /b EGW_GetCurrentCell( integer grid )
--/li /b EGW_GetDataCellValue( integer grid, integer row, integer col )
--/li /b EGW_SetCurrentCell( integer grid, integer row, integer col )
--/li /b EGW_LoadData ( integer id, sequence data, integer insert_position )
--/li /b EGW_ScrollToCell( integer grid, integer row, integer col )
--/li /b EGW_UnloadData( integer grid )
--</ul>

--/topic Scripting
--/proc EuSQL
--/desc Routines from EuSQL
--
--These routines from EuSQL may be used within a form's script.
--Please refer to the <a href="eusql.htm">EuSQL</a> documentation for full descriptions
--of the routines.
--<ul>
--/li /b get_sql_err( integer err_code ) Returns a description of the error
--/li /b parse_sql( sequence sql ) Compiles a SQL statement
--/li /b run_query( sequence query )  Runs a compiled query
--/li /b run_sql( sequence sql ) Compiles and runs a SQL statement
--/li /b select_table( sequence table ) Changes the current table
--/li /b set_parameter( sequence param_name, object val ) Sets the value of a parameter
--</ul>

    -- eusql
    { "set_parameter",          PROC,   2 },
    { "run_sql",                EUFUNC, 1 },
    { "run_query",              EUFUNC, 1 },
    { "parse_sql",              EUFUNC, 1 },
    { "select_table",           EUFUNC, 1 },
    { "get_sql_err",            EUFUNC, 1 },
    { "get_parameter",          EUFUNC, 1 }
    
    }

constant remove_these = {
--    "?", 
    "abort", "call", "c_func", "c_proc", "clear_screen",  
--	"close", 
    "command_line", "get_key", "get_pixel", "getc",
    "getenv", "gets", "machine_func", "machine_proc", "mem_copy",
    "mem_set", 
--	"open", 
    "peek", "peek4s", "peek4u", "platform",
    "pixel", "poke", "poke4", "position", 
--	"print", "printf",
    "profile", 
--	"puts", 
    "system", "system_exec","trace"
    }                                                 

constant edb_script = {
"constant",
"    MB_ABORTRETRYIGNORE = #02, --  Abort, Retry, Ignore",
"    MB_APPLMODAL = #00,       -- User must respond before doing something else",
"    MB_DEFAULT_DESKTOP_ONLY = #20000,    ",
"    MB_DEFBUTTON1 = #00,      -- First button is default button",
"    MB_DEFBUTTON2 = #100,      -- Second button is default button",
"    MB_DEFBUTTON3 = #200,      -- Third button is default button",
"    MB_DEFBUTTON4 = #300,   -- Fourth button is default button",
"    MB_HELP = #4000,            -- Windows 95: Help button generates help event",
"    MB_ICONASTERISK = #40,",
"    MB_ICONERROR = #10, ",
"    MB_ICONEXCLAMATION = #30, -- Exclamation-point appears in the box",
"    MB_ICONHAND = MB_ICONERROR,        -- A hand appears",
"    MB_ICONINFORMATION = MB_ICONASTERISK,-- Lowercase letter i in a circle appears",
"    MB_ICONQUESTION = #20,    -- A question-mark icon appears",
"    MB_ICONSTOP = MB_ICONHAND,",
"    MB_ICONWARNING = MB_ICONEXCLAMATION,",
"    MB_OK = #00,              -- Message box contains one push button: OK",
"    MB_OKCANCEL = #01,        -- Message box contains OK and Cancel",
"    MB_RETRYCANCEL = #05,     -- Message box contains Retry and Cancel",
"    MB_RIGHT = #80000,        -- Windows 95: The text is right-justified",
"    MB_RTLREADING = #100000,   -- Windows 95: For Hebrew and Arabic systems",
"    MB_SERVICE_NOTIFICATION = #40000, -- Windows NT: The caller is a service ",
"    MB_SETFOREGROUND = #10000,   -- Message box becomes the foreground window ",
"    MB_SYSTEMMODAL  = #1000,    -- All applications suspended until user responds",
"    MB_TASKMODAL = #2000,       -- Similar to MB_APPLMODAL ",
"    MB_YESNO = #04,           -- Message box contains Yes and No",
"    MB_YESNOCANCEL = #03,      -- Message box contains Yes, No, and Cancel",
"EGW_FIRST      = -1,",
"EGW_LAST       = -2,",
"EGW_REPLACE    = -3,",
"    Geometry = " & sprint(Geometry)
}

global procedure init_edbscript( integer id )
    activate_script( id )
    run_script( edb_script )
end procedure         

global procedure start_edbscript()
    integer add
    add = routine_id("add_routine")
    
    remove_routine( remove_these )
    for i = 1 to length(add_these) do
        call_proc( add, add_these[i] & routine_id(add_these[i][1]) )
    end for
end procedure