include eugrid.ew
--/////////////////////////////////////////////////////////////////////////////
--// Name:        edbutils.ew
--// Purpose:     include file for Euphoria Database Browser
--// Author:      Matt Lewis
--// Modified by:
--// Created:     2003-08-01
--// Copyright:   (c) Matt Lewis
--// Licence:     See documentation
--/////////////////////////////////////////////////////////////////////////////
include eusql.e
with trace
object void

-- store the info for the various grids that we
-- may need to keep track of (if not kept separate,
-- they can step on each other, with unpredictible
-- results)
sequence delayed_grid, delayed_row, delayed_col
delayed_grid = {}
delayed_row = {}
delayed_col = {}

global integer delay_on
delay_on = 0

-- have to scroll to a cell after it's been updated, or
-- it doesn't work properly, so I set up a timer to do this...
global procedure grid_delayed_scroll( integer self, atom event, sequence params )
    integer ix
    killTimer(self, 1 )
    ix = find( self, delayed_grid)
    if not ix then
        return
    end if
    void = EGW_ScrollToCell( self, delayed_row[ix], delayed_col[ix] )   
    void = EGW_SetCurrentCell( self, delayed_row[ix], delayed_col[ix] )
    void = EGW_SetDataRowFlag( self, delayed_row[ix], EGW_ROW_SELECTED, 1 )
    repaintWindow( self )
    delay_on = 0
end procedure

-- this sets up the timer for the above routine...
global procedure delayed_scroll( integer ctrl, atom row, atom col )
    integer ix
    ix = find(ctrl, delayed_grid)
    if not ix then
        delayed_grid &= ctrl
        delayed_row &= row
        delayed_col &= col
    else
        delayed_row[ix] = row
        delayed_col[ix] = col
    end if
    setHandler( ctrl, w32HTimer, routine_id("grid_delayed_scroll"))
    delay_on = 1
    setTimer( ctrl, 1, 10 )
end procedure

sequence grid_selected_row
grid_selected_row = {{},{}}

-- this will take the highlighting off of the row
-- in the grid.  select_grid must be called first,
-- or deselect_grid won't know the row number
global procedure deselect_grid( integer grid )
    atom row
    integer ix
    ix = find( grid, grid_selected_row[1] )
    if not ix then
        return
    end if

    row = grid_selected_row[2][ix]
    void = EGW_SetDataRowFlag( grid, row, EGW_ROW_SELECTED, 0 )
end procedure

-- scrolls to, selects and highlights the row, using
-- delayed_scroll
global procedure select_grid( integer grid, atom row )
    integer ix
    sequence cols
    ix = find( grid, grid_selected_row[1] )
    
    if row > power(2,31) then
        row = power(2,32)-row
    end if
    
    if not ix then
        grid_selected_row[1] &= grid
        grid_selected_row[2] &= row
    else
        grid_selected_row[2][ix] = row
    end if
    cols = EGW_EnumColumns( grid )
    delayed_scroll( grid, row, cols[1] )
end procedure

global function trim( sequence s )
    integer l
    l = length(s)
    while l and find(s[l], " \t\r\n") do
        l -= 1
    end while
    return s[1..l]
end function

global function ltrim( sequence s )
    integer l, i
    l = length(s)
    i = 1
    while i < l and find(s[i], " \t\r\n") do
        i += 1
    end while
    return s[i..l]
end function

global procedure clear_grid( integer grid )
    sequence cols
    
    void = EGW_LoadData( grid, {{}}, EGW_REPLACE )  
    void = EGW_DeleteDataRow( grid, 1 )

    cols = EGW_EnumColumns( grid )
    for i = length(cols) to 1 by -1 do
        void = EGW_DeleteColumn( grid, cols[i] )
    end for 
    
end procedure

constant DT_CHECK1 = {EUSQL_EU_SEQUENCE, EUSQL_EU_OBJECT, EUSQL_EU_ATOM, EUSQL_EU_INTEGER}
constant 
escape_code = "ntr\\\"",
escape_value = "\n\t\r\\\""

function remove_escape_codes( sequence s )
    integer ix, len, jx, flag
    ix = 1
    len = length(s)

    while ix < len do
        if s[ix] = '\\' then
            jx = find(s[ix+1],escape_code)
            if jx then
                s[ix] = escape_value[jx]
                s[ix+1..len-1] = s[ix+2..len]
                len -= 1
            else
                ix += 1
            end if
        else
            ix += 1
        end if
        
    end while
    return replace_all(s[1..len], "&&", "&")
end function

global function convert_eusql_data( object data, integer datatype )
    sequence val

    if atom(data) then
        return data
        data = sprint(data)
    elsif not length(data) then
        return data
    elsif datatype != EUSQL_EU_TEXT then
        val = value(data)
        if val[1] != GET_SUCCESS then
            if find(datatype, DT_CHECK1 ) then
                -- it's object or sequence , so plain text is ok
                -- or it's atom or integer, so we have type check
                -- issues, and returning the string will be fine
                return data
            else
                -- binary not formed properly, so return 0 to 
                -- force type check error
                return 0
            end if
        end if
        return val[2]
    else
        -- need to convert escape chars...
        data = remove_escape_codes( data )
--    	for i = 1 to length(escape_code) do
--    		data = replace_all( data, escape_code[i], escape_value[i] )
--    	end for
    end if
    return data
end function

global function convert_normal_data( object data )
    object test
    if atom(data) then 
        return data
    end if

    if length(data) and data[1] = '{' and data[length(data)] = '}' then
        data = value( data )
        data = data[2]
    else
        test = value( data )
        if test[1] = GET_SUCCESS then
            data = test[2]
        else
            -- need to convert escape chars...
            data = remove_escape_codes( data )
--	    	for i = 1 to length(escape_code) do
--	    		data = replace_all( data, escape_code[i], escape_value[i] )
--	    	end for
        end if
    end if
    return data
end function